<?php

namespace PublishPress;

/**
 * Attributes
 *
 * @package PressPermit
 * @author Kevin Behrens
 * @copyright Copyright (c) 2023 PublishPress
 *
 * Custom status capabilities are implemented by defining 'post_status' as a PublishPress Permissions Attribute.
 * 
 * For any attribute, the generic equivalent of "status" is "condition"
 * 
 */
class StatusCapabilities
{
    // Custom status capabilities are implemented by defining 'post_status' as an Attribute.

    var $attributes = [];               // attributes[attribute] = object with the following properties: conditions, label, taxonomies

                                        // phpcs:ignore Squiz.PHP.CommentedOutCode.Found
    var $condition_cap_map = [];        // condition_cap_map[basic_cap_name][attribute][condition] = condition_cap_name

                                        // phpcs:ignore Squiz.PHP.CommentedOutCode.Found
    var $condition_metacap_map = [];    // condition_metacap_map[object_type][basic_metacap_name][attribute][condition] = condition_cap_name
    var $all_custom_condition_caps = [];
    var $processed_statuses = [];
    var $all_privacy_caps;
    var $all_moderation_caps;

    private static $instance = null;

    public static function instance() {
        if (is_null(self::$instance)) {
            $class = apply_filters('publishpress_status_capabilities_class', 'PublishPress\StatusCapabilities');

            self::$instance = new $class();

            self::registerAttribute('post_status', 'post', ['label' => esc_html__('Post Status', 'pubishpress-status-capabilities')]);

            require_once(__DIR__ . '/CapabilityFilters.php');
            StatusCapabilities\CapabilityFilters::instance();
        }

        return self::$instance;
    }

    protected function __construct($args = [])
    {
        $defaults = [];
        foreach(array_keys($defaults) as $var) {
            $$var = $args[$var];
        }

        add_filter('publishpress_statuses_required_caps', [$this, 'fltStatusRequiredCaps'], 10, 5);

        add_action('init', [$this, 'act_post_stati_prep'], 48);

        add_action('init', [$this, 'actForceDistinctPostCaps'], 50);

        // phpcs:ignore Squiz.PHP.CommentedOutCode.Found
        // Permissions Pro's StatusesHooksAdmin::act_process_conditions() hooks to presspermit_conditions_loaded, which is triggered by this wp_loaded handler
        add_action('wp_loaded', [$this, 'act_late_registrations']);

        add_filter('presspermit_get_post_statuses', [$this, 'fltGetPostStatuses'], 10, 5);

        add_filter('publishpress_status_properties', [$this, 'fltStatusProperties']);

        add_filter('publishpress_statuses_admin_columns', [$this, 'fltStatusesColumns']);

        add_filter('publishpress_statuses_edit_status_tabs', [$this, 'fltStatusEditTabs'], 10, 2);

        add_action('publishpress_statuses_edit_status_tab_content', [$this, 'actStatusEditTabSections'], 10, 2);

        add_action('publishpress_statuses_edit_status', [$this, 'actHandleEditStatus'], 10, 2);

        add_filter('publishpress_statuses_block_editor_args', [$this, 'flt_publishpress_statuses_block_editor_args'], 10, 2);
    }

    // @todo: review
    function act_post_stati_prep()
    {
        global $wp_post_statuses;

        // set default properties
        foreach (array_keys($wp_post_statuses) as $status) {
            if (!isset($wp_post_statuses[$status]->moderation))
                $wp_post_statuses[$status]->moderation = false;
        }
    }

    function actForceDistinctPostCaps()
    {
        global $wp_post_types;

        $generic_caps = ['post' => ['set_posts_status' => 'set_posts_status'], 'page' => ['set_posts_status' => 'set_posts_status']];

        // post types which are enabled for PP filtering must have distinct type-related cap definitions
        foreach (
            array_intersect(
                get_post_types(
                    ['public' => true, 'show_ui' => true], 
                    'names', 
                    'or'
                ), 
                array_keys(self::getEnabledPostTypes())
            ) as $post_type
        ) {
            if ('post' == $post_type) {
                $type_caps['set_posts_status'] = 'set_posts_status';
            } else {
                $type_caps['set_posts_status'] = str_replace('_post', "_$post_type", 'set_posts_status');
            }

            $wp_post_types[$post_type]->cap = (object)array_merge((array)$wp_post_types[$post_type]->cap, $type_caps);

            $plural_type = self::getPlural($post_type);

            self::addTypeCaps($type_caps);

            foreach (self::getPostStatuses(['moderation' => true, 'post_type' => $post_type]) as $status_name) {
                $cap_property = "set_{$status_name}_posts";
                $wp_post_types[$post_type]->cap->$cap_property = str_replace("_posts", "_{$plural_type}", $cap_property);
            }
        }
    }

    private static function getPlural($post_type) {
        global $wp_post_types;

        if (class_exists('PublishPress\Permissions\Capabilities')) {
            $plural_type = \PublishPress\Permissions\Capabilities::getPlural($post_type, $wp_post_types[$post_type]);
        
        } elseif (function_exists('_cme_get_plural')) {
            $plural_type = _cme_get_plural($post_type, $wp_post_types[$post_type]);
        
        } else {
            $plural_type = $post_type . 's';
        }

        return $plural_type;
    }

    private static function addTypeCaps($type_caps) {
        global $cme_cap_helper;

        if (function_exists('presspermit')) {
            $cap_helper = presspermit()->capDefs();

        } elseif (!empty($cme_cap_helper)) {
            $cap_helper = $cme_cap_helper;
        }

        if ($cap_helper) {
            $cap_helper->all_type_caps = array_merge($cap_helper->all_type_caps, array_fill_keys($type_caps, true));
        }
    }

    // late registration of statuses for PublishPress compat (PublishPress hooks to 'init' action at priority 1000)
    function act_late_registrations()
    {
        global $wp_post_statuses, $current_user;

        // @todo: eliminate ?
        foreach (['pending', 'future'] as $status) {
            if (!empty($wp_post_statuses[$status])) {
                $wp_post_statuses[$status]->moderation = true;
            }
        }

        $is_administrator = is_super_admin() || current_user_can('administrator');

        $skip_metacaps = !self::customStatusPostMetaPermissions()
        && (
            (!$is_administrator && empty($current_user->allcaps['pp_moderate_any']))
            && (!is_admin() || !isset($_SERVER['SCRIPT_NAME']) || false == strpos(sanitize_text_field($_SERVER['SCRIPT_NAME']), 'admin.php')        // phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.NonceVerification.Required
            || !isset($_REQUEST['page']) || !in_array($_REQUEST['page'], ['pp-capabilities', 'pp-capabilities-settings', 'publishpress-statuses'])  // phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.NonceVerification.Required
            )
        );

        if (!$skip_metacaps) {
            // register each custom post status as an attribute condition with mapped caps
            foreach (self::getPostStatuses(['moderation' => true], 'object') as $status => $status_obj) {
                if (!empty($status_obj->moderation)) {
                    if (isset($status_obj->capability_status) && ('' === $status_obj->capability_status)) {
                        continue;
                    }

                    $_status = (isset($status_obj->capability_status) && ($status != $status_obj->capability_status) 
                    && !empty($wp_post_statuses[$status_obj->capability_status])) 
                    ? $status_obj->capability_status 
                    : $status;

                    $metacap_map = ['edit_post' => "edit_{$_status}_posts", 'delete_post' => "delete_{$_status}_posts"];

                    self::registerCondition('post_status', $status, [
                        'label' => $status_obj->label,
                        'metacap_map' => $metacap_map,
                        'cap_map' => [
                            'set_posts_status' => "set_posts_{$_status}",                         
                            'edit_others_posts' => "edit_others_{$status}_posts",
                            'delete_others_posts' => "delete_others_{$status}_posts"
                        ]
                    ]);
                }
            }
        }

        do_action('publishpress_status_capabilities_loaded');
    
        $this->process_status_caps();
    }

    /*
    * args:
    *  label = translated string
    */
    public static function registerAttribute($attribute, $source_name = 'post', $args = [])
    {
        $defaults = ['label' => $attribute, 'taxonomies' => []];
        $args = array_merge($defaults, $args);
        $args['conditions'] = [];
        $args['source_name'] = $source_name;

        if (empty(self::instance()->attributes[$attribute])) {
            self::instance()->attributes[$attribute] = (object)$args;
        }
    }

    // args:
    //   label = translated string
    //   cap_map = ['base_cap_property' => restriction_cap_pattern] where restriction_cap_pattern may contain "_posts" 
    //      (will be converted to plural name of obj type)
    //
    //   metacap_map = ['meta_cap' => restriction_cap_pattern]
    //
    //   exemption_cap = base cap property corresponding to a capability whose presence in a role indicates the role 
    //      should be credited with all caps for this status 
    //      (i.e. if a role has $cap->publish_posts, it also has all 'restricted_submission' caps) 
    public static function registerCondition($attribute, $condition, $args = [])
    {
        $defaults = ['label' => $condition, 'cap_map' => [], 'metacap_map' => []];
        $args = array_merge($defaults, $args);

        if (!isset(self::instance()->attributes[$attribute])) {
            return;
        }

        $args['name'] = $condition;
        self::instance()->attributes[$attribute]->conditions[$condition] = (object)$args;
    }

    // @todo: still needed?
    function fltStatusProperties($status) {
        $statuses = $this->fltGetPostStatuses([$status->name => $status]);

        return array_pop($statuses);
    }

    function fltGetPostStatuses($statuses, $status_args = [], $return_args = [], $operator = '', $function_args = []) {
        $context = (!empty($return_args['context'])) ? $return_args['context'] : '';

        $status_cap_status = array_intersect_key((array) get_option('presspermit_status_capability_status'), $statuses);
        
        foreach ($statuses as $k => $status) {
            if (!is_object($status)) {
                continue;
            }

            if (in_array($status->name, ['draft', 'publish', 'private', 'future'])) {
                continue;

            // Workflow statuses, including Pending, default to standard capabilities
            } elseif (!empty($status->moderation) || ('pending' == $status->name)) {
                $statuses[$k]->capability_status = (isset($status_cap_status[$status->name])) ? $status_cap_status[$status->name] : '';

            // Visibility statuses default to custom capabilities
            } elseif (!empty($status->private) && empty($status->_builtin)) {
                $statuses[$k]->capability_status = (isset($status_cap_status[$status->name])) ? $status_cap_status[$status->name] : $status->name;
            }
        }

        return $statuses;
    }

    function fltStatusRequiredCaps($status_caps, $cap_type, $status_name, $post_type, $args = []) {
        switch ($cap_type) {
            case 'set_status':
                $type_obj = get_post_type_object($post_type);

                if ($check_caps = $this->getConditionCaps(
                    $type_obj->cap->set_posts_status, 
                    $post_type, 
                    'post_status', 
                    $status_name
                )) {
                    $status_caps = $check_caps;
                }

                break;
        }

        return $status_caps;
    }

    function is_metacap($caps)
    {
        return (bool)array_intersect((array)$caps, ['read_post', 'read_page', 'edit_post', 'edit_page', 'delete_post', 'delete_page']);
    }

    public static function postStatusHasCustomCaps($status)
    {
        return !empty(self::instance()->attributes['post_status']->conditions[$status]);
    }

    public static function customStatusPostMetaPermissions($post_type = '', $post_status = '', $args = []) {
        if (!$attributes = self::instance()) {
            return false;
        }

        if (is_object($post_status)) {
            $status_obj = $post_status;
            $post_status = strval($post_status->name);
        } else {
            $status_obj = get_post_status_object($post_status);
        }

        if ($post_status && !self::postStatusHasCustomCaps($post_status)) {
            return false;
        }

        if ($post_type) {
            if ($status_obj) {
                if (!empty($status_obj->post_type) && !in_array($post_type, $status_obj->post_type)) {
                    return false;
                }
            }
        }

        return self::presspermitStatusControlActive() || !defined('PUBLISHPRESS_CAPS_PRO_VERSION') 
        || get_option('cme_custom_status_postmeta_caps') || !empty($args['ignore_capabilities_option']);
    }

    public static function presspermitStatusControlActive() {
        return function_exists('presspermit') && presspermit()->moduleActive('status-control') && presspermit()->moduleActive('collaboration');
    }

    public static function getEnabledPostTypes() {
        if (function_exists('presspermit')) {
            return presspermit()->getEnabledPostTypes([], 'object');

        } elseif (function_exists('cme_get_assisted_post_types')) {
            $post_types = cme_get_assisted_post_types();

        } else {
            $option_name = (defined('PPC_VERSION') && !defined('PRESSPERMIT_VERSION')) ? 'pp_enabled_post_types' : 'presspermit_enabled_post_types';
            $enabled = (array) get_option( $option_name, array( 'post' => true, 'page' => true ) );

            $post_types = array_intersect( $post_types, array_keys( array_filter( $enabled ) ) );
        }

        $type_objects = [];

        foreach($post_types as $post_type) {
            if (post_type_exists($post_type)) {
                $type_objects[$post_type] = get_post_type_object($post_type);
            }
        }

        return $type_objects;
    }

    function process_status_caps()
    {
        global $wp_post_statuses;

        if (empty($this->all_custom_condition_caps)) {
            $this->all_custom_condition_caps = ['post' => []];
            $this->all_privacy_caps = ['post' => []];
            $this->all_moderation_caps = ['post' => []];
        }

        if (isset($this->attributes['post_status'])) {
            foreach (array_keys($this->attributes['post_status']->conditions) as $cond) {
                if (isset($this->processed_statuses[$cond])) continue;

                $this->processed_statuses[$cond] = true;

                $status_obj = (!empty($wp_post_statuses[$cond])) ? $wp_post_statuses[$cond] : false;

                foreach ($this->getEnabledPostTypes() as $object_type => $type_obj) {
                    if (!empty($wp_post_statuses[$cond]->post_type) && !in_array($object_type, $wp_post_statuses[$cond]->post_type)) {
                        continue;
                    }

                    // convert 'edit_restricted_posts' to 'edit_restricted_pages', etc.
                    $plural_name = (isset($type_obj->plural_name)) ? $type_obj->plural_name : $object_type . 's';

                    // Map condition caps to post meta caps( 'edit_post', 'delete_post', etc. ) because:
                    //  (1) mapping to expanded caps is problematic b/c for private posts, 'edit_private_posts' is required but 'edit_posts' is not
                    //  (2) WP converts type-specific meta caps back to basic metacap equivalent before calling 'map_meta_cap'
                    foreach (
                        $this->attributes['post_status']->conditions[$cond]->metacap_map 
                        as $base_cap_property => $condition_cap_pattern
                    ) {
                        // If the type object has "edit_restricted_posts" defined, use it.
                        $replacement_cap = (isset($type_obj->cap->$condition_cap_pattern)) 
                        ? $type_obj->cap->$condition_cap_pattern 
                        : str_replace('_posts', "_{$plural_name}", $condition_cap_pattern);

                        $this->condition_metacap_map[$object_type][$base_cap_property]['post_status'][$cond] = $replacement_cap;

                        switch ($base_cap_property) {
                            case 'read_post':
                                $type_cap = PRESSPERMIT_READ_PUBLIC_CAP;
                                break;
                            case 'edit_post':
                                $type_cap = $type_obj->cap->edit_posts;
                                break;
                            case 'delete_post':
                                if (isset($type_obj->cap->delete_posts))
                                    $type_cap = $type_obj->cap->delete_posts;
                                else
                                    $type_cap = str_replace('edit_', 'delete_', $type_obj->cap->edit_posts);
                                break;
                            default:
                                $type_cap = $base_cap_property;
                        }
                        $this->all_custom_condition_caps[$object_type][$replacement_cap] = $type_cap;

                        if (!empty($status_obj->private))
                            $this->all_privacy_caps[$object_type][$replacement_cap] = $type_cap;

                        if (!empty($status_obj->moderation))
                            $this->all_moderation_caps[$object_type][$replacement_cap] = $type_cap;
                    }

                    foreach (
                        $this->attributes['post_status']->conditions[$cond]->cap_map 
                        as $base_cap_property => $condition_cap_pattern
                    ) {
                        // If the type object has "edit_restricted_posts" defined, use it.
                        $replacement_cap = (isset($type_obj->cap->$condition_cap_pattern)) 
                        ? $type_obj->cap->$condition_cap_pattern 
                        : str_replace('_posts', "_{$plural_name}", $condition_cap_pattern);

                        $cap_name = (isset($type_obj->cap->$base_cap_property)) 
                        ? $type_obj->cap->$base_cap_property 
                        : $base_cap_property;

						// Previous versions defined edit_others capability as "edit_others_{$status_name}_attachments" for all post types
                        if (!isset($this->condition_cap_map[$cap_name]['post_status'][$cond]) || defined('PRESSPERMIT_LEGACY_STATUS_CAP_DEFINITIONS')) {
                            $this->condition_cap_map[$cap_name]['post_status'][$cond] = $replacement_cap;
                        }

                        $this->all_custom_condition_caps[$object_type][$replacement_cap] = $cap_name;

                        if (!empty($status_obj->private)) {
                            $this->all_privacy_caps[$object_type][$replacement_cap] = $cap_name;
                        }

                        if (!empty($status_obj->moderation)) {
                            $this->all_moderation_caps[$object_type][$replacement_cap] = $cap_name;
                        }
                    }
                } // end foreach object type
            } // end foreach condition

            $moderation_statuses = get_post_stati(['moderation' => true, '_builtin' => false], 'names');

            // Support use of PublishPress-defined status change capability where type-specific status capabilities are not enabled
            foreach($this->condition_cap_map as $cap_name => $conditions) {
                if (0 === strpos($cap_name, 'set_') && !empty($conditions['post_status'])) {
                    foreach($moderation_statuses as $status) {
                        if (!isset($this->condition_cap_map[$cap_name]['post_status'][$status]) && !in_array($status, ['draft', 'future'])) {
                            $_status = str_replace('-', '_', $status);
                            $this->condition_cap_map[$cap_name]['post_status'][$status] = "status_change_{$_status}";
                        }
                    }
                }
            }
        }
    }

    function getConditionCaps($reqd_caps, $object_type, $attribute, $conditions, $args = [])
    {
        $defaults = ['merge_caps' => []];
        $args = array_merge($defaults, $args);
        foreach (array_keys($defaults) as $var) {
            $$var = $args[$var];
        }

        if ($merge_caps) {
            $merge_caps = array_map('sanitize_key', (array) $merge_caps);
        }

        $cond_caps = [];

        $reqd_caps = (array)$reqd_caps;

        foreach ($reqd_caps as $base_cap) {
            foreach ((array)$conditions as $cond) {
                if (!empty($this->condition_cap_map[$base_cap][$attribute][$cond])) {
                    $cond_caps[] = $this->condition_cap_map[$base_cap][$attribute][$cond];
                }

                if (!empty($this->condition_metacap_map[$object_type][$base_cap][$attribute][$cond])) {
                    $cond_caps[] = $this->condition_metacap_map[$object_type][$base_cap][$attribute][$cond];
                }
            }
        }

        if ($merge_caps) {
            $cond_caps = array_merge($cond_caps, $merge_caps);

            // If a status-specific edit_others_{$status}_posts capability is defined, don't also assign / require edit_others_posts
            foreach(['edit_others_posts', 'delete_others_posts'] as $base_cap) {
                if (!empty($this->condition_cap_map[$base_cap][$attribute][$cond])) {
                    if ($type_obj = get_post_type_object($object_type)) {
                        if (!empty($type_obj->cap->$base_cap)) {
                            $cond_caps = array_diff($cond_caps, [$type_obj->cap->$base_cap]);
                        }
                    }
                }
            }
        }

        return array_unique($cond_caps);
    }

    public static function getPostStatuses($args, $return = 'names', $operator = 'and', $function_args = [])
    {
        if (isset($args['post_type'])) {
            $post_type = $args['post_type'];
            unset($args['post_type']);
            $stati = get_post_stati($args, 'object', $operator);

            foreach ($stati as $status => $obj) {
                if (!empty($obj->post_type) && !array_intersect((array)$post_type, (array)$obj->post_type))
                    unset($stati[$status]);
            }

            $statuses = ('names' == $return) ? array_keys($stati) : $stati;
        } else {
            $statuses = get_post_stati($args, $return, $operator);
        }

        return apply_filters('presspermit_get_post_statuses', $statuses, $args, $return, $operator, $function_args);
    }

    function fltStatusesColumns($cols) {
        if (self::customStatusPostMetaPermissions()) {
            if (!empty($cols['description'])) {
                $col_descript = $cols['description'];
                unset($cols['description']);
            }

            $cols['enabled'] = esc_html__('Post Access');

            if (!empty($col_descript)) {
                $cols['description'] = $col_descript;
            }
        }

        return $cols;
    }

    function fltStatusEditTabs($tabs, $status_name) {
        if (self::customStatusPostMetaPermissions()) {
            if ($status_obj = get_post_status_object($status_name)) {
                if (empty($status_obj->publish) && empty($status_obj->private) && !in_array($status_name, ['draft', 'future', 'publish'])) {
                    $tabs['post_access'] = __('Post Access', 'publishpress-status-capabilities');
                }
            }
        }

        return $tabs;
    }

    function actStatusEditTabSections($status, $default_tab) {
        require_once(__DIR__ . '/StatusAdmin.php');
        new \PublishPress\StatusCapabilities\StatusAdmin();

        do_action('presspermit_statuses_edit_status_tab', 'post_access', $status, $default_tab);
    }

    function actHandleEditStatus($status_name, $args) {
        require_once(__DIR__ . '/StatusSave.php');
        \PublishPress\StatusCapabilities\StatusSave::save($status_name);
    }

    function flt_publishpress_statuses_block_editor_args($args, $filter_args) {
        if (function_exists('presspermit')) {
            $user = presspermit()->getUser();
        } else {
            global $current_user;
            $user = $current_user;
        }

        $filter_args = (array) $filter_args;

        $defaults = ['post_id' => 0, 'post_type' => '', 'status' => ''];

        foreach ($defaults as $var => $default_val) {
            $$var = (isset($filter_args[$var])) ? $filter_args[$var] : $default_val;
        }
        
        $attributes = self::instance();

        if ($check_caps = $attributes->getConditionCaps('edit_post', $post_type, 'post_status', $status)) {
            if (array_diff($check_caps, array_keys($user->allcaps))) {
                $args["redirectURL{$status}"] = admin_url("edit.php?post_type={$post_type}&pp_submission_done={$status}");
            }
        }
        
        return $args;
    }
}
